'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _has = require('lodash/has');

var _has2 = _interopRequireDefault(_has);

var _isSchema = require('./util/isSchema');

var _isSchema2 = _interopRequireDefault(_isSchema);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function callOrConcat(schema) {
  if (typeof schema === 'function') {
    return schema;
  }

  return base => base.concat(schema);
}

class Conditional {
  constructor(refs, options) {
    const { is } = options;
    let { then, otherwise } = options;

    this.refs = [].concat(refs);

    then = callOrConcat(then);
    otherwise = callOrConcat(otherwise);

    if (typeof options === 'function') {
      this.fn = options;
    } else {
      if (!(0, _has2.default)(options, 'is')) {
        throw new TypeError('`is:` is required for `when()` conditions');
      }

      if (!options.then && !options.otherwise) {
        throw new TypeError('either `then:` or `otherwise:` is required for `when()` conditions');
      }

      const isFn = typeof is === 'function' ? is : (...values) => values.every(value => value === is);

      this.fn = function fn(...values) {
        const currentSchema = values.pop();
        const option = isFn(...values) ? then : otherwise;

        return option(currentSchema);
      };
    }
  }

  getValue(parent, context) {
    const values = this.refs.map(r => r.getValue(parent, context));

    return values;
  }

  resolve(ctx, values) {
    const schema = this.fn.apply(ctx, values.concat(ctx));

    if (schema !== undefined && !(0, _isSchema2.default)(schema)) {
      throw new TypeError('conditions must return a schema object');
    }

    return schema || ctx;
  }
}

exports.default = Conditional;