'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = NumberSchema;

var _inherits = require('./util/inherits');

var _inherits2 = _interopRequireDefault(_inherits);

var _mixed = require('./mixed');

var _mixed2 = _interopRequireDefault(_mixed);

var _locale = require('./locale');

var _isAbsent = require('./util/isAbsent');

var _isAbsent2 = _interopRequireDefault(_isAbsent);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/* eslint-disable no-param-reassign */
const isNaN = value => value !== +value;
const isInteger = val => (0, _isAbsent2.default)(val) || val === (val | 0); // eslint-disable-line no-bitwise

function NumberSchema() {
  if (!(this instanceof NumberSchema)) {
    return new NumberSchema();
  }

  _mixed2.default.call(this, { type: 'number' });

  this.withMutation(() => {
    this.transform(function transform(value) {
      if (this.isType(value)) return value;

      const parsed = parseFloat(value);
      if (this.isType(parsed)) return parsed;

      return NaN;
    });
  });
}

(0, _inherits2.default)(NumberSchema, _mixed2.default, {

  _typeCheck(value) {
    if (value instanceof Number) {
      value = value.valueOf();
    }

    return typeof value === 'number' && !isNaN(value);
  },

  min(min, msg) {
    return this.test({
      name: 'min',
      exclusive: true,
      params: { min },
      message: msg || _locale.number.min,
      test(value) {
        return (0, _isAbsent2.default)(value) || value >= this.resolve(min);
      }
    });
  },

  max(max, msg) {
    return this.test({
      name: 'max',
      exclusive: true,
      params: { max },
      message: msg || _locale.number.max,
      test(value) {
        return (0, _isAbsent2.default)(value) || value <= this.resolve(max);
      }
    });
  },

  positive(msg) {
    return this.min(0, msg || _locale.number.positive);
  },

  negative(msg) {
    return this.max(0, msg || _locale.number.negative);
  },

  integer(msg) {
    msg = msg || _locale.number.integer;

    return this.test('integer', msg, isInteger);
  },

  truncate() {
    return this.transform(value => !(0, _isAbsent2.default)(value) ? value | 0 : value); // eslint-disable-line no-bitwise
  },

  round(method) {
    const avail = ['ceil', 'floor', 'round', 'trunc'];
    method = method && method.toLowerCase() || 'round';

    // this exists for symemtry with the new Math.trunc
    if (method === 'trunc') {
      return this.truncate();
    }

    if (avail.indexOf(method.toLowerCase()) === -1) {
      throw new TypeError(`Only valid options for round() are: ${avail.join(', ')}`);
    }

    return this.transform(value => !(0, _isAbsent2.default)(value) ? Math[method](value) : value);
  }
});