'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; }; /* eslint-disable no-param-reassign */


exports.default = ObjectSchema;

var _propertyExpr = require('property-expr');

var _camelCase = require('lodash/camelCase');

var _camelCase2 = _interopRequireDefault(_camelCase);

var _has = require('lodash/has');

var _has2 = _interopRequireDefault(_has);

var _isArray = require('lodash/isArray');

var _isArray2 = _interopRequireDefault(_isArray);

var _isPlainObject = require('lodash/isPlainObject');

var _isPlainObject2 = _interopRequireDefault(_isPlainObject);

var _mapKeys = require('lodash/mapKeys');

var _mapKeys2 = _interopRequireDefault(_mapKeys);

var _omit = require('lodash/omit');

var _omit2 = _interopRequireDefault(_omit);

var _snakeCase = require('lodash/snakeCase');

var _snakeCase2 = _interopRequireDefault(_snakeCase);

var _transform = require('lodash/transform');

var _transform2 = _interopRequireDefault(_transform);

var _mixed = require('./mixed');

var _mixed2 = _interopRequireDefault(_mixed);

var _locale = require('./locale');

var _sortFields = require('./util/sortFields');

var _sortFields2 = _interopRequireDefault(_sortFields);

var _sortByKeyOrder = require('./util/sortByKeyOrder');

var _sortByKeyOrder2 = _interopRequireDefault(_sortByKeyOrder);

var _inherits = require('./util/inherits');

var _inherits2 = _interopRequireDefault(_inherits);

var _makePath = require('./util/makePath');

var _makePath2 = _interopRequireDefault(_makePath);

var _runValidations = require('./util/runValidations');

var _runValidations2 = _interopRequireDefault(_runValidations);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function unknown(ctx, value) {
  const known = Object.keys(ctx.fields);
  return Object.keys(value).filter(key => known.indexOf(key) === -1);
}

function ObjectSchema(spec) {
  if (!(this instanceof ObjectSchema)) {
    return new ObjectSchema(spec);
  }

  _mixed2.default.call(this, { type: 'object',
    default() {
      const dft = (0, _transform2.default)(this._nodes, (obj, key) => {
        obj[key] = this.fields[key].default ? this.fields[key].default() : undefined;
      }, {});

      return Object.keys(dft).length === 0 ? undefined : dft;
    }
  });

  this.fields = Object.create(null);
  this._nodes = [];
  this._excludedEdges = [];

  this.withMutation(() => {
    this.transform(function coerce(value) {
      if (typeof value === 'string') {
        try {
          value = JSON.parse(value);
        } catch (err) {
          value = null;
        }
      }
      if (this.isType(value)) {
        return value;
      }
      return null;
    });

    if (spec) {
      this.shape(spec);
    }
  });
}

(0, _inherits2.default)(ObjectSchema, _mixed2.default, {

  _typeCheck(value) {
    return (0, _isPlainObject2.default)(value) || typeof value === 'function';
  },

  _cast(_value, options = {}) {
    let value = _mixed2.default.prototype._cast.call(this, _value, options);

    // should ignore nulls here
    if (value === undefined) {
      return this.default();
    }

    if (!this._typeCheck(value)) {
      return value;
    }

    const fields = this.fields;
    const strip = this._option('stripUnknown', options) === true;
    const extra = Object.keys(value).filter(v => this._nodes.indexOf(v) === -1);
    const props = this._nodes.concat(extra);

    const innerOptions = _extends({}, options, {
      parent: {}, // is filled during the transform below
      __validating: false
    });

    value = (0, _transform2.default)(props, (obj, prop) => {
      let field = fields[prop];
      const exists = (0, _has2.default)(value, prop);

      if (field) {
        const strict = field._options && field._options.strict;

        // safe to mutate since this is fired in sequence
        innerOptions.path = _makePath2.default`${options.path}.${prop}`;
        innerOptions.value = value[prop];

        field = field.resolve(innerOptions);

        if (field._strip === true) {
          return;
        }

        const fieldValue = !options.__validating || !strict ? field.cast(value[prop], innerOptions) : value[prop];

        if (fieldValue !== undefined) {
          obj[prop] = fieldValue;
        }
      } else if (exists && !strip) {
        obj[prop] = value[prop];
      }
    }, innerOptions.parent);

    return value;
  },

  _validate(_value, opts = {}) {
    const errors = [];
    let originalValue = opts.originalValue != null ? opts.originalValue : _value;

    const abortEarly = this._option('abortEarly', opts);
    const recursive = this._option('recursive', opts);
    const sync = this._option('sync', opts);

    opts = _extends({}, opts, { __validating: true, originalValue });

    return _mixed2.default.prototype._validate.call(this, _value, opts).catch((0, _runValidations.propagateErrors)(abortEarly, errors)).then(value => {
      if (!recursive || !(0, _isPlainObject2.default)(value)) {
        // only iterate though actual objects
        if (errors.length) throw errors[0];
        return value;
      }

      originalValue = originalValue || value;

      const validations = this._nodes.map(key => {
        const path = _makePath2.default`${opts.path}.${key}`;
        const field = this.fields[key];

        const innerOptions = _extends({}, opts, {
          path,
          parent: value,
          originalValue: originalValue[key]
        });

        if (field) {
          // inner fields are always strict:
          // 1. this isn't strict so the casting will also have cast inner values
          // 2. this is strict in which case the nested values weren't cast either
          innerOptions.strict = true;

          if (field.validate) {
            return field.validate(value[key], innerOptions);
          }
        }

        return true;
      });

      return (0, _runValidations2.default)({
        validations,
        value,
        errors,
        abortEarly,
        sync,
        path: opts.path,
        sort: (0, _sortByKeyOrder2.default)(this.fields)
      });
    });
  },

  concat(schema) {
    const next = _mixed2.default.prototype.concat.call(this, schema);

    next._nodes = (0, _sortFields2.default)(next.fields, next._excludedEdges);

    return next;
  },

  shape(schema, excludes = []) {
    const next = this.clone();
    const fields = Object.assign(next.fields, schema);

    if (!(0, _isArray2.default)(excludes[0])) {
      excludes = [excludes];
    }

    next.fields = fields;

    if (excludes.length) {
      const keys = excludes.map(([first, second]) => `${first}-${second}`);

      next._excludedEdges = next._excludedEdges.concat(keys);
    }

    next._nodes = (0, _sortFields2.default)(fields, next._excludedEdges);

    return next;
  },

  from(from, to, alias) {
    const fromGetter = (0, _propertyExpr.getter)(from, true);

    return this.transform(obj => {
      let newObj = obj;

      if (obj == null) {
        return obj;
      }

      if ((0, _has2.default)(obj, from)) {
        newObj = alias ? _extends({}, obj) : (0, _omit2.default)(obj, from);
        newObj[to] = fromGetter(obj);
      }

      return newObj;
    });
  },

  noUnknown(noAllow = true, message = _locale.object.noUnknown) {
    if (typeof noAllow === 'string') {
      message = noAllow;
      noAllow = true;
    }

    const next = this.test({
      name: 'noUnknown',
      exclusive: true,
      message,
      test(value) {
        return value == null || !noAllow || unknown(this.schema, value).length === 0;
      }
    });

    if (noAllow) {
      next._options.stripUnknown = true;
    }

    return next;
  },

  transformKeys(fn) {
    return this.transform(obj => obj && (0, _mapKeys2.default)(obj, (_, key) => fn(key)));
  },

  camelCase() {
    return this.transformKeys(_camelCase2.default);
  },

  snakeCase() {
    return this.transformKeys(_snakeCase2.default);
  },

  constantCase() {
    return this.transformKeys(key => (0, _snakeCase2.default)(key).toUpperCase());
  }
});