'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; }; /* eslint-disable no-param-reassign */


exports.default = createValidation;

var _mapValues = require('lodash/mapValues');

var _mapValues2 = _interopRequireDefault(_mapValues);

var _getPromise = require('./getPromise');

var _getPromise2 = _interopRequireDefault(_getPromise);

var _Reference = require('../Reference');

var _Reference2 = _interopRequireDefault(_Reference);

var _ValidationError = require('../ValidationError');

var _ValidationError2 = _interopRequireDefault(_ValidationError);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _objectWithoutProperties(obj, keys) { var target = {}; for (var i in obj) { if (keys.indexOf(i) >= 0) continue; if (!Object.prototype.hasOwnProperty.call(obj, i)) continue; target[i] = obj[i]; } return target; }

const formatError = _ValidationError2.default.formatError;

function resolveParams(oldParams, newParams, resolve) {
  return (0, _mapValues2.default)(_extends({}, oldParams, newParams), resolve);
}

function createErrorFactory(_ref) {
  let { value, label, resolve, originalValue } = _ref,
      opts = _objectWithoutProperties(_ref, ['value', 'label', 'resolve', 'originalValue']);

  return function createError({
    path = opts.path,
    message = opts.message,
    type = opts.name,
    params
  } = {}) {
    params = _extends({
      path,
      value,
      originalValue,
      label
    }, resolveParams(opts.params, params, resolve));

    return Object.assign(new _ValidationError2.default(formatError(message, params), value, path, type), { params });
  };
}

function createValidation(options) {
  const { name, message, test, params } = options;

  function validate(_ref2) {
    let {
      value,
      path,
      label,
      options: validateOptions,
      sync: validateSync,
      originalValue
    } = _ref2,
        rest = _objectWithoutProperties(_ref2, ['value', 'path', 'label', 'options', 'sync', 'originalValue']);

    const sync = options.sync || validateSync;
    const parent = validateOptions.parent;
    const resolve = val => _Reference2.default.isRef(val) ? val.getValue(parent, validateOptions.context) : val;

    const createError = createErrorFactory({
      message,
      path,
      value,
      originalValue,
      params,
      label,
      resolve,
      name
    });

    const ctx = _extends({
      path,
      parent,
      type: name,
      createError,
      resolve,
      options: validateOptions
    }, rest);

    return (0, _getPromise2.default)(sync).resolve(test.call(ctx, value)).then(validOrError => {
      if (_ValidationError2.default.isError(validOrError)) {
        throw validOrError;
      } else if (!validOrError) {
        throw createError();
      }
    });
  }

  validate.TEST_NAME = name;
  validate.TEST_FN = test;
  validate.TEST = options;

  return validate;
}

module.exports.createErrorFactory = createErrorFactory;