'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = reach;

var _propertyExpr = require('property-expr');

var _has = require('lodash/has');

var _has2 = _interopRequireDefault(_has);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/* eslint-disable no-param-reassign */
const trim = part => part.substr(0, part.length - 1).substr(1);

function reach(obj, path, value, context) {
  let parent;
  let lastPart;

  // if only one "value" arg then use it for both
  context = context || value;

  (0, _propertyExpr.forEach)(path, (_part, isBracket, isArray) => {
    const part = isBracket ? trim(_part) : _part;

    if (isArray || (0, _has2.default)(obj, '_subType')) {
      // we skipped an array: foo[].bar
      const idx = isArray ? parseInt(part, 10) : 0;

      obj = obj.resolve({ context, parent, value })._subType;

      if (value) {
        if (isArray && idx >= value.length) {
          throw new Error(`Yup.reach cannot resolve an array item at index: ${_part}, in the path: ${path}. ` + 'because there is no value at that index. ');
        }

        value = value[idx];
      }
    }

    if (!isArray) {
      obj = obj.resolve({ context, parent, value });

      if (!(0, _has2.default)(obj, 'fields') || !(0, _has2.default)(obj.fields, part)) {
        throw new Error(`The schema does not contain the path: ${path}. ` + `(failed at: ${lastPart} which is a type: "${obj._type}") `);
      }

      obj = obj.fields[part];

      parent = value;
      value = value && value[part];
      lastPart = isBracket ? `[${_part}]` : `.${_part}`;
    }
  });

  if (obj) {
    obj = obj.resolve({ context, parent, value });
  }

  return obj;
}