'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.propagateErrors = propagateErrors;
exports.settled = settled;
exports.collectErrors = collectErrors;
exports.default = runValidations;

var _getPromise = require('./getPromise');

var _getPromise2 = _interopRequireDefault(_getPromise);

var _ValidationError = require('../ValidationError');

var _ValidationError2 = _interopRequireDefault(_ValidationError);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _objectWithoutProperties(obj, keys) { var target = {}; for (var i in obj) { if (keys.indexOf(i) >= 0) continue; if (!Object.prototype.hasOwnProperty.call(obj, i)) continue; target[i] = obj[i]; } return target; }

const unwrapError = (errors = []) => errors.inner && errors.inner.length ? errors.inner : [].concat(errors);

function scopeToValue({ validations, value, sync }) {
  return (0, _getPromise2.default)(sync).all(validations).catch(err => {
    if (err.name === 'ValidationError') {
      err.value = value; // eslint-disable-line no-param-reassign
    }
    throw err;
  }).then(() => value);
}

/**
 * If not failing on the first error, catch the errors
 * and collect them in an array
 */
function propagateErrors(abortEarly, errors) {
  return abortEarly ? null : err => {
    errors.push(err);
    return err.value;
  };
}

function settled(promises, sync) {
  const settle = promise => promise.then(value => ({ fulfilled: true, value }), value => ({ fulfilled: false, value }));

  return (0, _getPromise2.default)(sync).all(promises.map(settle));
}

function collectErrors({
  validations,
  value,
  path,
  errors,
  sort,
  sync
}) {
  return settled(validations, sync).then(results => {
    const nestedErrors = results.filter(r => !r.fulfilled).reduce((arr, { value: error }) => {
      // we are only collecting validation errors
      if (!_ValidationError2.default.isError(error)) {
        throw error;
      }
      return arr.concat(error);
    }, []);

    if (sort) {
      nestedErrors.sort(sort);
    }

    // show parent errors after the nested ones: name.first, name
    const allErrors = nestedErrors.concat(unwrapError(errors));

    if (allErrors.length) {
      throw new _ValidationError2.default(allErrors, value, path);
    }

    return value;
  });
}

function runValidations(_ref) {
  let { abortEarly } = _ref,
      options = _objectWithoutProperties(_ref, ['abortEarly']);

  if (abortEarly) {
    return scopeToValue(options);
  }

  return collectErrors(options);
}